! module with the Hamiltonian function and the Hamilton's equations
module hamiltonian

  use parameters
  use coordinates
  implicit none

contains

!***************************************************************************************

  ! Hamiltonian of the (N+1)-body problem (+J2)
  ! implemented in canonical heliocentric/planetocentric cartesian coordinates
  function Ham(p,x)

     implicit none

     type(hamparam),intent(inOut):: p
     double precision,intent(in):: x(6*p%Nbod)
     double precision:: Ham

     double precision:: r,r2,r3,rji2,s,s2
     double precision:: y(6,p%Nbod)
     double precision:: d(3)
     integer:: i, j

     !-----------------------------------------------
     Ham = 0.d0
 
     !-----------------------------------------------
     do i=1,p%Nbod
        ! keplerian
        y(:,i) = x(6*i-5:6*i)
        r2=DOT_PRODUCT(y(1:3,i),y(1:3,i))
        r=sqrt(r2)
        Ham = Ham + DOT_PRODUCT(y(4:6,i),y(4:6,i))/(2.d0*p%bi(i))
        Ham = Ham - p%GmJmibi(i)/r
        !--------------------------------------------
        ! oblateness J2
        s=y(3,i)/r
        s2=s**2
        r3=r*r2
        Ham = Ham + p%GmJmiRJ2J2(i)/r3*(1.5d0*s2-0.5d0)
     enddo
     !-----------------------------------------------
     ! mutual perturbations between celestial bodies
     do i=1,p%Nbod
        do j = i+1,p%Nbod
           Ham = Ham + DOT_PRODUCT(y(4:6,i),y(4:6,j))/p%m0
           ! direct
           d    = y(1:3,j)-y(1:3,i)
           rji2 = DOT_PRODUCT(d,d)
           Ham = Ham - p%Gmimj(i,j)/sqrt(rji2)
        enddo
     enddo

  end function Ham

!***************************************************************************************

  ! right-hand side of the equations of motion of the (N+1)-body problem (+J2)
  ! implemented in canonical heliocentric/planetocrentric cartesian coordinates
  function DHam(p,x)

     implicit none

     type(hamparam),intent(inOut):: p
     double precision,intent(in):: x(6*p%Nbod)
     double precision:: DHam(6,p%Nbod)

     double precision:: r2,r3,r5,rji2,rji3,s,s2
     double precision:: y(6,p%Nbod)
     double precision:: d(3),aux(3)
     integer:: i, j

     !-----------------------------------------------
     DHam = 0.d0

     !-----------------------------------------------
     do i=1,p%Nbod
        ! keplerian
        y(:,i) = x(6*i-5:6*i)
        r2=DOT_PRODUCT(y(1:3,i),y(1:3,i))
        r3=sqrt(r2)*r2
        DHam(1:3,i) =   y(4:6,i)/p%bi(i)
        DHam(4:6,i) = - p%GmJmibi(i)*y(1:3,i)/r3
        !--------------------------------------------
        ! oblateness J2
        s=y(3,i)/sqrt(r2)
        s2=s**2
        r5=r2*r3
        aux(1) = p%GmJmiRJ2J2(i)/r5*(-7.5d0*s2+1.5d0)
        aux(2) = aux(1)*y(2,i)
        aux(3) = (aux(1) + 3.d0*p%GmJmiRJ2J2(i)/r5)*y(3,i)
        aux(1) = aux(1)*y(1,i)
        DHam(4:6,i) = DHam(4:6,i) - aux
     enddo
     !-----------------------------------------------
     ! mutual perturbations between celestial bodies
     do i=1,p%Nbod
        do j = 1,p%Nbod
           ! indirect
           if(j.ne.i)then
              DHam(1:3,i) = DHam(1:3,i) + y(4:6,j)/p%m0
           endif
           ! direct
           if(j.gt.i)then
              d    = y(1:3,j)-y(1:3,i)
              rji2 = DOT_PRODUCT(d,d)
              rji3 = sqrt(rji2)*rji2
              aux  = p%Gmimj(i,j)*d/rji3
              DHam(4:6,i) = DHam(4:6,i) + aux
              DHam(4:6,j) = DHam(4:6,j) - aux
           endif
        enddo
     enddo
     !-----------------------------------------------

  end function DHam

!***************************************************************************************

end module hamiltonian
